@@ -24,6 +24,7 @@
 #include "wirelessdevice.h"
 
 #include <QDBusConnection>
+#include <QTimer>
 
 #include "connectionsettings.h"
 #include "../manager.h"
@@ -215,8 +216,14 @@ void FakeNetwork::unregisterService()
 
 QDBusObjectPath FakeNetwork::ActivateConnection(const QDBusObjectPath &connection, const QDBusObjectPath &device, const QDBusObjectPath &specific_object)
 {
-    ActiveConnection *newActiveConnection = new ActiveConnection(this);
     QString newActiveConnectionPath = QString("/org/kde/fakenetwork/ActiveConnection/") + QString::number(m_activeConnectionsCounter++);
+    ActiveConnection *newActiveConnection = new ActiveConnection(this);
+    newActiveConnection->addDevice(device);
+    newActiveConnection->setActiveConnectionPath(newActiveConnectionPath);
+    newActiveConnection->setConnection(connection);
+    newActiveConnection->setSpecificObject(specific_object);
+    newActiveConnection->setState(NetworkManager::ActiveConnection::Activating);
+
     m_activeConnections.insert(QDBusObjectPath(newActiveConnectionPath), newActiveConnection);
     QDBusConnection::sessionBus().registerObject(newActiveConnectionPath, newActiveConnection, QDBusConnection::ExportScriptableContents);
 
@@ -227,24 +234,6 @@ QDBusObjectPath FakeNetwork::ActivateConnection(const QDBusObjectPath &connectio
     map.insert(QLatin1Literal("ActivatingConnection"), QVariant::fromValue(QDBusObjectPath(newActiveConnectionPath)));
     Q_EMIT PropertiesChanged(map);
 
-    newActiveConnection->addDevice(device);
-    newActiveConnection->setActiveConnectionPath(newActiveConnectionPath);
-    newActiveConnection->setConnection(connection);
-    newActiveConnection->setSpecificObject(specific_object);
-    newActiveConnection->setState(NetworkManager::ActiveConnection::Activating);
-
-    map.clear();
-    const QList<QDBusObjectPath> deviceList { device };
-    map.insert(QLatin1Literal("Devices"), QVariant::fromValue<QList<QDBusObjectPath> >(deviceList));
-    map.insert(QLatin1Literal("Connection"), QVariant::fromValue<QDBusObjectPath>(connection));
-    if (!specific_object.path().isEmpty()) {
-        map.insert(QLatin1Literal("SpecificObject"), QVariant::fromValue<QDBusObjectPath>(connection));
-    }
-    map.insert(QLatin1Literal("State"), NetworkManager::ActiveConnection::Activating);
-    QDBusMessage message = QDBusMessage::createSignal(newActiveConnectionPath, QLatin1Literal("org.kde.fakenetwork.Connection.Active"), QLatin1Literal("PropertiesChanged"));
-    message << map;
-    QDBusConnection::sessionBus().send(message);
-
     Device *usedDevice = static_cast<Device *>(QDBusConnection::sessionBus().objectRegisteredAt(device.path()));
     if (usedDevice) {
         m_activatedDevice = usedDevice->devicePath();
@@ -281,6 +270,10 @@ void FakeNetwork::updateConnectingState()
             QDBusMessage message = QDBusMessage::createSignal(activeConnection->activeConnectionPath(), QLatin1Literal("org.kde.fakenetwork.Connection.Active"), QLatin1Literal("PropertiesChanged"));
             message << activeConnectionMap;
             QDBusConnection::sessionBus().send(message);
+
+            QDBusMessage message2 = QDBusMessage::createSignal(activeConnection->activeConnectionPath(), QLatin1Literal("org.kde.fakenetwork.Connection.Active"), QLatin1Literal("StateChanged"));
+            message2 << (uint)2 << (uint)1; // NM_ACTIVE_CONNECTION_STATE_ACTIVATED << NM_ACTIVE_CONNECTION_STATE_REASON_NONE
+            QDBusConnection::sessionBus().send(message2);
         }
         // TODO: set dhcp4Config, dhcp6Config, ip4Config, ip6Config
         // IP Interface is usually same as interface
@@ -339,6 +332,10 @@ void FakeNetwork::DeactivateConnection(const QDBusObjectPath &active_connection)
         message << activeConnectionMap;
         QDBusConnection::sessionBus().send(message);
 
+        QDBusMessage message2 = QDBusMessage::createSignal(activeConnection->activeConnectionPath(), QLatin1Literal("org.kde.fakenetwork.Connection.Active"), QLatin1Literal("StateChanged"));
+        message2 << (uint)4 << (uint)2; // NM_ACTIVE_CONNECTION_STATE_DEACTIVATED << NM_ACTIVE_CONNECTION_STATE_REASON_USER_DISCONNECTED
+        QDBusConnection::sessionBus().send(message2);
+
         Device *device = m_devices.value(activeConnection->devices().first());
         if (device) {
             m_deactivatedDevice = device->devicePath();
@@ -377,6 +374,10 @@ void FakeNetwork::updateDeactivatingState()
         message << activeConnectionMap;
         QDBusConnection::sessionBus().send(message);
 
+        QDBusMessage message2 = QDBusMessage::createSignal(activeConnection->activeConnectionPath(), QLatin1Literal("org.kde.fakenetwork.Connection.Active"), QLatin1Literal("StateChanged"));
+        message2 << (uint)3 << (uint)2; // NM_ACTIVE_CONNECTION_STATE_DEACTIVATING << NM_ACTIVE_CONNECTION_STATE_REASON_USER_DISCONNECTED
+        QDBusConnection::sessionBus().send(message2);
+
         removeActiveConnection(QDBusObjectPath(activeConnection->activeConnectionPath()));
     }
 
